/*!
 * Copyright (C) 2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Log: optoin.c,v $
 * Revision 1.1  2002/06/12 11:33:50  harald
 * First check in
 *
 */

#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/event.h>
#include <sys/print.h>

#include <dev/spidigio.h>

/*!
 * \example optoin/optoin.c
 *
 * This sample demonstrates how to query the  
 * SPI Hexadecimal Isolated Input Board.
 */

static volatile u_long ival;
static HANDLE iocq;

/*
 * Scans for digital input changes. 
 */
THREAD(InputScanner, arg)
{
    u_char num = (u_char)((u_short)arg);
    u_long rval;

    rval = ival = SpiDigitalGet(num);
    NutEventPost(&iocq);

    for(;;) 
        if((rval = SpiDigitalGet(num)) != ival) {
            ival = rval;
            NutEventPost(&iocq);
        }
        else
            NutThreadYield();
}

/*
 * Main application routine. 
 *
 * Nut/OS automatically calls this entry after initialization.
 */
THREAD(NutMain, arg)
{
    u_char ni;
    u_char no;
    u_long baud = 115200;
    
    /*
     * Initialize debug UART device and
     * print banner.
     */
    outp(BV(RXEN) | BV(TXEN), UCR);
    NutDeviceIOCtl(0, UART_SETSPEED, &baud);
    NutPrintString_P(0, PSTR("\r\nDigital Opto Input Demo\r\n"));
    
    /*
     * Keep looking for inputs.
     */
    for(;;) {
        /*
         * Initialize the SPI to the digital I/O boards.
         * The call returns the number of inputs and
         * outputs detected.
         */
        SpiDigitalInit(&ni, &no);
        NutPrintFormat(0, "Found %u digital inputs and %u digital outputs\r\n", ni, no);
        if(ni)
            break;
        NutSleep(500);
    }

    NutThreadCreate("iscan", InputScanner, (void *)(ni + no), 512);

    for(;;) {
        NutEventWait(&iocq, 0);
        NutPrintFormat(0, "I/O: %08lX\r\n", ival);
    }
}
