/*!
 * Copyright (C) 2001-2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Log: portdio.c,v $
 * Revision 1.5  2002/06/26 17:29:03  harald
 * First pre-release with 2.4 stack
 *
 * Revision 1.4  2002/06/12 10:56:16  harald
 * *** empty log message ***
 *
 * Revision 1.3  2002/06/04 19:12:37  harald
 * *** empty log message ***
 *
 * Revision 1.2  2002/05/08 16:02:31  harald
 * First Imagecraft compilation
 *
 */

/*!
 * \example portdio/portdio.c
 *
 * TCP server for Port D I/O.
 *
 * Program Ethernut with portdio.rom and enter
 *
 * telnet x.x.x.x 12345
 *
 * two times on a command prompt, replacing x.x.x.x 
 * with the IP address of your ethernut board. 
 * Enter help for a list of available commands.
 *
 * This will start two telnet session. Enter query
 * on the first will show the current port status.
 * Then enter wait on this session, which will
 * hang until the port status changes. On the
 * second telnet session enter set1 to set the
 * first output bit.
 */

#include <string.h>

#include <dev/nicrtl.h>

#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/print.h>

#include <netinet/sostream.h>
#include <arpa/inet.h>

/*
 * Process client requests.
 */
void ProcessRequests(NUTDEVICE *sostream)
{
    u_char buff[128];
    int n;
    int stat = -1;

    NutPrintString(sostream, "200 Welcome to portdio. Type help to get help.\r\n");
    for(;;) {
        NutPrintFlush(sostream);

        /*
         * Read a line from the client. Ignore
         * blank lines.
         */
        if((n = NutDeviceGetLine(sostream, buff, sizeof(buff))) < 0)
            break;
        if(n == 0)
            continue;

        /*
         * Memory info.
         */
        if(strncmp(buff, "memory", strlen(buff)) == 0) {
            NutPrintFormat(sostream, "210 %u bytes RAM free\r\n", NutHeapAvailable());
            continue;
        }

        /*
         * List threads.
         */
        if(strncmp(buff, "threads", strlen(buff)) == 0) {
            NUTTHREADINFO *tdp;
            NUTTIMERINFO *tnp;

            NutPrintString(sostream, "220 List of threads with name,state,prio,stack,mem,timeout follows\r\n");
            for(tdp = nutThreadList; tdp; tdp = tdp->td_next) {
                NutPrintString(sostream, tdp->td_name);
                NutPrintString(sostream, "\t");
                switch(tdp->td_state) {
                case TDS_TERM:
                    NutPrintString(sostream, "\tTerm\t");
                    break;
                case TDS_RUNNING:
                    NutPrintString(sostream, "\tRun\t");
                    break;
                case TDS_READY:
                    NutPrintString(sostream, "\tReady\t");
                    break;
                case TDS_SLEEP:
                    NutPrintString(sostream, "\tSleep\t");
                    break;
                }
                NutPrintInteger(sostream, tdp->td_priority, 10, 0, 0);
                NutPrintString(sostream, "\t");
                NutPrintInteger(sostream, (u_short)tdp->td_sp - (u_short)tdp->td_memory, 10, 0, 0);
                if(*((u_long *)tdp->td_memory) != DEADBEEF)
                    NutPrintString(sostream, "\tCorrupted\t");
                else
                    NutPrintString(sostream, "\tOK\t");

                if((tnp = (NUTTIMERINFO *)tdp->td_timer) != 0) 
                    NutPrintInteger(sostream, tnp->tn_ticks_left, 10, 0, 0);
                else
                    NutPrintString(sostream, "None");
                NutPrintString(sostream, "\r\n");
            }
            NutPrintString(sostream, ".\r\n");
            continue;
        }

        /*
         * List timer.
         */
        if(strncmp(buff, "timers", strlen(buff)) == 0) {
            NUTTIMERINFO *tnp;

            NutPrintString(sostream, "221 List of timers with ticks left and interval follows\r\n");
            for(tnp = nutTimerList; tnp; tnp = tnp->tn_next) {
                NutPrintInteger(sostream, tnp->tn_ticks_left, 10, 0, 0);
                if(tnp->tn_ticks) {
                    NutPrintString(sostream, "\t");
                    NutPrintInteger(sostream, tnp->tn_ticks, 10, 0, 0);
                }
                else 
                    NutPrintString(sostream, "\tOneshot");
                NutPrintString(sostream, "\r\n");
            }
            NutPrintString(sostream, ".\r\n");
            continue;
        }

        /*
         * Port status.
         */
        if(strncmp(buff, "query", strlen(buff)) == 0) {
            stat = inp(PIND);
            NutPrintString(sostream, "210 ");
            NutPrintInteger(sostream, stat, 2, 8, FMTFLG_ZERO);
            NutPrintString(sostream, "\r\n");
            continue;
        }

        /*
         * Reset output bit.
         */
        if(strlen(buff) > 1 && strncmp(buff, "reset", strlen(buff) - 1) == 0) {
            int mask = 0;
            switch(buff[strlen(buff) - 1]) {
            case '1':
                mask = 0x10;
                break;
            case '2':
                mask = 0x20;
                break;
            case '3':
                mask = 0x40;
                break;
            case '4':
                mask = 0x80;
                break;
            }
            if(mask) {
                outp(inp(PORTD) & ~mask, PORTD);
                NutPrintString(sostream, "210 OK\r\n");
            }
            else
                NutPrintString(sostream, "410 Bad pin\r\n");
            continue;
        }

        /*
         * Set output bit.
         */
        if(strlen(buff) > 1 && strncmp(buff, "set", strlen(buff) - 1) == 0) {
            int mask = 0;
            switch(buff[strlen(buff) - 1]) {
            case '1':
                mask = 0x10;
                break;
            case '2':
                mask = 0x20;
                break;
            case '3':
                mask = 0x40;
                break;
            case '4':
                mask = 0x80;
                break;
            }
            if(mask) {
                outp(inp(PORTD) | mask, PORTD);
                NutPrintString(sostream, "210 OK\r\n");
            }
            else
                NutPrintString(sostream, "410 Bad pin\r\n");
            continue;
        }

        /*
         * wait for status change.
         */
        if(strncmp(buff, "wait", strlen(buff)) == 0) {
            while(stat == inp(PIND))
                NutThreadYield();
            stat = inp(PIND);
            NutPrintString(sostream, "210 ");
            NutPrintInteger(sostream, stat, 2, 8, FMTFLG_ZERO);
            NutPrintString(sostream, "\r\n");
            continue;
        }

        /*
         * Help.
         */
        NutPrintString(sostream, "400 List of commands follows\r\n");
        NutPrintString(sostream, "memory\tQueries number of RAM bytes free\r\n");
        NutPrintString(sostream, "query\tQuery digital i/o status\r\n");
        NutPrintString(sostream, "reset#\tSet output bit 1 to 4 to 0\r\n");
        NutPrintString(sostream, "set#\tSet output bit 1 to 4 to 1\r\n");
        NutPrintString(sostream, "threads\tLists all created threads\r\n");
        NutPrintString(sostream, "timers\tLists all running timers\r\n");
        NutPrintString(sostream, "wait\tWaits for digital i/o change\r\n");
        NutPrintString(sostream, ".\r\n");
    }
}

/*
 * Init Port D
 */
void init_dio(void)
{
    /*
     * Upper four pins are output pins.
     */
    outp(0xf0, DDRD);

    /*
     * Outputs to low and inputs pulled up.
     */
    outp(0x0f, PORTD);
}

void service(void)
{
    TCPSOCKET *sock;
    NUTDEVICE *sostream;

    /*
     * Loop endless for connections.
     */
    for(;;) {
        /*
         * Create a socket.
         */
        sock = NutTcpCreateSocket();

        /*
         * Listen on port 12345. If we return,
         * we got a client.
         */
        NutTcpAccept(sock, 12345);

        /*
         * Create a device from the socket,
         * so we can use NutPrintFormat() and
         * other device goodies.
         */
        sostream = NutSoStreamCreate(sock);

        /*
         * Process client requests.
         */
        ProcessRequests(sostream);

        /*
         * Destroy our device.
         */
        NutSoStreamDestroy(sostream);

        /*
         * Close our socket.
         */
        NutTcpCloseSocket(sock);
    }
}

THREAD(service_thread, arg)
{
    for(;;)
        service();
}

/*
 * Main application routine. 
 *
 * Nut/OS automatically calls this entry after initialization.
 */
THREAD(NutMain, arg)
{
    /*
     * Initialize digital I/O.
     */
    init_dio();

    /*
     * Register Realtek controller at address 8300 hex
     * and interrupt 5.
     */
    NutRegisterDevice(&devEth0, 0x8300, 5);

    /*
     * Configure lan interface. 
     */
    NutNetAutoConfig("eth0");

    /*
     * Start another service thread to allow
     * two concurrent connections.
     */
    NutThreadCreate("sback", service_thread, 0, 1384);

    for(;;)
        service();
}
