/*!
 * Copyright (C) 2001-2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Log: tcpc.c,v $
 * Revision 1.7  2002/06/26 17:29:05  harald
 * First pre-release with 2.4 stack
 *
 * Revision 1.6  2002/06/12 10:58:55  harald
 * *** empty log message ***
 *
 * Revision 1.5  2002/06/04 19:12:47  harald
 * *** empty log message ***
 *
 * Revision 1.4  2002/05/08 16:02:32  harald
 * First Imagecraft compilation
 *
 */

/*!
 * \example tcpc/tcpc.c
 *
 * Simple TCP echo client. Connects an echo
 * server running on a specified host and
 * a specified port.
 *
 * You need to modify ECHOSERVERIP and
 * ECHOSERVERPORT definitions.
 *
 */
#define ECHOSERVERIP    "192.168.1.1"
#define ECHOSERVERPORT  7

#include <string.h>

#include <dev/nicrtl.h>

#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/print.h>

#include <netinet/sostream.h>
#include <arpa/inet.h>

static u_char buff[1024];

/*
 * Main application routine. 
 *
 * Nut/OS automatically calls this entry after initialization.
 */
THREAD(NutMain, arg)
{
    TCPSOCKET *sock;

    /*
     * Register Realtek controller at address 8300 hex
     * and interrupt 5 and configure lan interface.
     */
    NutRegisterDevice(&devEth0, 0x8300, 5);
    NutNetAutoConfig("eth0");
    
    for(;;) {
        /*
         * Create a socket.
         */
        sock = NutTcpCreateSocket();

        /*
         * Connect an echo server.
         */
        if(NutTcpConnect(sock, inet_addr(ECHOSERVERIP), ECHOSERVERPORT) == 0) {

            /*
             * Fill buffer with letter A.
             */
            memset(buff, 'A', 383);
            buff[383] = 0;

            /*
             * Send the string and receive the response.
             */
            if(NutTcpSend(sock, buff, 384) > 0)
                NutTcpReceive(sock, buff, sizeof(buff));
        }

        /*
         * Close our socket and take a nap.
         */
        NutTcpCloseSocket(sock);
        NutSleep(1000);
    }
}
