#ifndef _DEV_ATAREGS_H_
#define _DEV_ATAREGS_H_

/*
 * Copyright (C) 2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 */

/*
 * Physical interface.
 */

/*
 * Least significant byte of the 
 * drive's bidirectional data bus.
 */
#define ATA_DDLSB_PIN  PINA
#define ATA_DDLSB_PORT PORTA
#define ATA_DDLSB_DDR  DDRA

/*
 * Drive chip select 0, used to select
 * the command block registers.
 */
#define ATA_CS1FX_BIT   PB6
#define ATA_CS1FX_PORT  PORTB
#define ATA_CS1FX_DDR   DDRB

/*
 * Drive reset. Must be pulled low for at least
 * 25 microseconds after voltage levels have
 * stabilized during power on.
 */
#define ATA_RESET_BIT   PB7         /*!< \brief Reset bit. */
#define ATA_RESET_PORT  PORTB       /*!< \brief Reset port output register. */
#define ATA_RESET_DDR   DDRB        /*!< \brief Reset port direction register. */

/*
 * Drive address bus, used to access a
 * register or or data port in the drive.
 * Shared with CPU address bits 8-10.
 */
#define ATA_DA_MASK     (BV(PC0) | BV(PC1) | BV(PC2))
#define ATA_DA_PORT     PORTC
#ifdef DDRC
#define ATA_DA_DDR      DDRC
#endif

/*
 * Drive I/O read strobe. The falling edge 
 * enables data from a register or the data 
 * port of the drive onto the data bus.
 */
#define ATA_DIOR_BIT    PC3
#define ATA_DIOR_PORT   PORTC
#ifdef DDRC
#define ATA_DIOR_DDR    DDRC
#endif

/*
 * Drive I/O write strobe. The rising edge 
 * clocks data from the data bus into a 
 * register or the port of the drive.
 */
#define ATA_DIOW_BIT    PC4
#define ATA_DIOW_PORT   PORTC
#ifdef DDRC
#define ATA_DIOW_DDR    DDRC
#endif

/*
 * Most significant byte of the 
 * drive's bidirectional data bus.
 */
#define ATA_DDMSB_PIN   PIND
#define ATA_DDMSB_PORT  PORTD
#define ATA_DDMSB_DDR   DDRD

/*
 * Drive chip select 1 used to select
 * the control block registers.
 */
#define ATA_CS3FX_BIT   PE4
#define ATA_CS3FX_PORT  PORTE
#define ATA_CS3FX_DDR   DDRE

/*
 * Drive interrupt request. Only set low when
 * the drive has a pending interrupt, the drive
 * is selected and the IEN bit in the device
 * control register is cleared. If the drive
 * is not selected or the IEN bit is set, then
 * this line is in high impedance state.
 */
#define ATA_INTRQ_BIT   PE7
#define ATA_INTRQ_PIN   PINE
#define ATA_INTRQ_DDR   DDRE

/*
 * Drive 16-bit I/O. Set low by the drive when
 * the data port has been addressed and the drive
 * is prepared to send or receive a 16-bit data
 * word. This is an open collector output.
 */
#define ATA_IOCS16_BIT  PF6
#define ATA_IOCS16_PIN  PINF
#ifdef DDRF
#define ATA_IOCS16_DDR  DDRF
#endif

/*
 * I/O channel ready. Set low by the drive to
 * extend host transfer cycle. Otherwise this
 * line is in high impedance state.
 */
#define ATA_IORDY_BIT   PF7
#define ATA_IORDY_PIN   PINF
#ifdef DDRF
#define ATA_IORDY_DDR   DDRF
#endif


/*
 * Logical interface.
 * Note that chip selects are active low.
 */

/*
 * Control block registers.
 */

/*
 * Alternate status register.
 */
#define ATA_ASTAT_REG   0x06

/*
 * Device control register.
 */
#define ATA_DC_REG      0x06

#define ATA_DC_IEN      0x02
#define ATA_DC_SRST     0x04

/*
 * Drive address register. Contains the inverted
 * drive select and head select address.
 */
#define ATA_DA_REG      0x07

#define ATA_DA_DS0      0x01
#define ATA_DA_DS1      0x02
#define ATA_DA_HS0      0x04
#define ATA_DA_HS1      0x08
#define ATA_DA_HS2      0x10
#define ATA_DA_HS3      0x20
#define ATA_DA_WTG      0x40

/*
 * Command block registers.
 */

/*
 * 16-bit data register.
 */
#define ATA_DATA_REG    0x80

/*
 * Error register.
 */
#define ATA_ERR_REG     0x81

#define ATA_ERR_AMNF
#define ATA_ERR_TKONF
#define ATA_ERR_ABRT
#define ATA_ERR_MCR
#define ATA_ERR_IDNF
#define ATA_ERR_MC
#define ATA_ERR_UNC
#define ATA_ERR_BRK

/*
 * Optional feature register.
 */
#define ATA_FEAT_REG    0x81

/*
 * Sector count register.
 */
#define ATA_SCNT_REG    0x82

/*
 * CHS address registers.
 */
#define ATA_CYLO_REG    0x84
#define ATA_CYHI_REG    0x85
#define ATA_DRHD_REG    0x86
#define ATA_SECT_REG    0x83

#define ATA_DRHD_HS0    0x01
#define ATA_DRHD_HS1    0x02
#define ATA_DRHD_HS2    0x04
#define ATA_DRHD_HS3    0x08
#define ATA_DRHD_DRV    0x10

/*
 * LBA address registers.
 */
#define ATA_LBA0_REG    0x83
#define ATA_LBA1_REG    0x84
#define ATA_LBA2_REG    0x85
#define ATA_LBA3_REG    0x86


/*
 * Status register.
 */
#define ATA_STAT_REG    (BV(ATA_CS3FX_BIT) | 7)

#define ATA_STAT_ERR    0x01    /*!< Indicates an error on the last command. */
#define ATA_STAT_IDX    0x02    /*!< Set once per disc revolution. */
#define ATA_STAT_CORR   0x04    /*!< Indicates correction of corrupted data. */
#define ATA_STAT_DRQ    0x08    /*!< Data request bit. */
#define ATA_STAT_DSC    0x10    /*!< Drive seek complete. */
#define ATA_STAT_DWF    0x20    /*!< Drive write fault. */
#define ATA_STAT_DRDY   0x40    /*!< Drive ready. */
#define ATA_STAT_BSY    0x80    /*!< Drive busy. */

/*
 * Command register.
 */
#define ATA_CMD_REG     0x87


#endif
