#ifndef _SYS_DEVICE_H_
#define _SYS_DEVICE_H_

/*
 * Copyright (C) 2001-2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 */

/*
 * $Log: device.h,v $
 * Revision 1.12  2002/08/08 17:24:21  harald
 * Using time constants by KU
 *
 * Revision 1.11  2002/07/03 16:45:41  harald
 * Using GCC 3.2
 *
 * Revision 1.10  2002/06/26 17:29:28  harald
 * First pre-release with 2.4 stack
 *
 */
#include <pgmspace.h>

#include <sys/types.h>

/*!
 * \file sys/device.h
 * \brief Nut/OS device definitions.
 */

/*!
 * \addtogroup xgDevice
 */
/*@{*/


// wait times for emulation and reality
// has to be overworked

#ifndef __EMULATION__
#define WAIT5		5
#define WAIT50		50
#define WAIT100		100
#define WAIT250		250
#define WAIT500		500
#else
#define WAIT5		1
#define WAIT50		5
#define WAIT100		10
#define WAIT250		25
#define WAIT500		50
#endif




#define IFTYP_RAM       0   /*!< \brief RAM device */
#define IFTYP_ROM       1   /*!< \brief ROM device */
#define IFTYP_STREAM    2   /*!< \brief Stream device */
#define IFTYP_NET       3   /*!< \brief Net device */

/*!
 * \brief Device structure type.
 */
typedef struct _NUTDEVICE NUTDEVICE;

/*!
 * \struct _NUTDEVICE device.h sys/device.h
 * \brief Device structure.
 */
struct _NUTDEVICE {
    NUTDEVICE *dev_next;       /*!< \brief Link to the next device structure. */
    u_char  dev_name[9];    /*!< \brief Unique device name. */
    u_char  dev_type;       /*!< \brief Type of interface. 
                             * May be any of the following:
                             * - IFTYP_RAM
                             * - IFTYP_ROM
                             * - IFTYP_STREAM
                             * - IFTYP_NET
                             */
    u_short dev_base;       /*!< \brief Hardware base address. 
                             *   Will be set by calling NutRegisterDevice(). 
                             *   On some device drivers this address may
                             *   be fixed.
                             */
    u_char  dev_irq;        /*!< \brief Interrupt registration number. 
                             *   Will be set by calling NutRegisterDevice(). 
                             *   On some device drivers the interrupt may
                             *   be fixed.
                             */
    void   *dev_icb;        /*!< \brief Interface control block.
                             *   With stream devices, this points to the
                             *   IFSTREAM structure and with network
                             *   devices this is a pointer to the IFNET
                             *   structure.
                             */
    void   *dev_dcb;        /*!< \brief Driver control block.
                             *   Points to a device specific information block.
                             */
    int   (*dev_init)(NUTDEVICE *);   /*!< \brief Driver initialization routine. 
                                    *   With stream devices this is called during
                                    *   NutDeviceOpen(). For network devices
                                    *   this routine is called within NutNetIfConfig().
                                    */
    int   (*dev_ioctl)(NUTDEVICE *, int, void *);  /*!< \brief Driver control function. 
                                                 *   Used to modify or query device 
                                                 *   specific settings.
                                                 */
};
/*@}*/


/*!
 * \addtogroup xgUARTIOCTL
 */
/*@{*/

#define UART_SETSPEED       0x0101  /*!< \brief Set baudrate. */
#define UART_GETSPEED       0x0102  /*!< \brief Query baudrate. */
#define UART_SETDATABITS    0x0103  /*!< \brief Set number of data bits. */
#define UART_GETDATABITS    0x0104  /*!< \brief Set number of data bits. */
#define UART_SETPARITY      0x0105  /*!< \brief Set parity. */
#define UART_GETPARITY      0x0106  /*!< \brief Query parity. */
#define UART_SETSTOPBITS    0x0107  /*!< \brief Set number of stop bits. */
#define UART_GETSTOPBITS    0x0108  /*!< \brief Query number of stop bits. */
#define UART_SETSTATUS      0x0109  /*!< \brief Set status. */
#define UART_GETSTATUS      0x010a  /*!< \brief Query status. */
#define UART_SETREADTIMEOUT 0x010b  /*!< \brief Set read timeout. */
#define UART_GETREADTIMEOUT 0x010c  /*!< \brief Query read timeout. */
#define UART_SETWRITETIMEOUT 0x010d /*!< \brief Set write timeout. */
#define UART_GETWRITETIMEOUT 0x010e /*!< \brief Query write timeout. */

#define DIO_SETSTATUS       0x0209  /*!< \brief Set digital output status. */
#define DIO_GETSTATUS       0x020a  /*!< \brief Query digital input status. */
#define DIO_WAITSTATUS      0x020b  /*!< \brief Wait for digital input status change. */

/*@}*/

/*!
 * \addtogroup xgUARTStatus
 */
/*@{*/
#define UART_FRAMINGERROR   0x00000001  /*!< \brief Framing error. */
#define UART_OVERRUNERROR   0x00000002  /*!< \brief Overrun error. */
#define UART_RXBUFFEREMPTY  0x00000004  /*!< \brief Receiver buffer empty. */
#define UART_TXBUFFEREMPTY  0x00000008  /*!< \brief Transmitter buffer empty. */
/*@}*/

extern NUTDEVICE *nutDeviceList;

extern int NutRegisterDevice(NUTDEVICE *dev, u_short base, u_char irq);
extern NUTDEVICE *NutDeviceLookup(CONST char *name);
extern NUTDEVICE *NutDeviceOpen(CONST char *name);
extern int NutDeviceClose(NUTDEVICE *dev);
extern int NutDeviceRead(NUTDEVICE *dev, void *data, int size);
extern int NutDeviceReadTran(NUTDEVICE *dev, void *data, int size);
extern int NutDeviceGetLine(NUTDEVICE *dev, void *data, int size);
extern int NutDeviceWrite(NUTDEVICE *dev, CONST void *data, int len);
extern int NutDeviceWriteTran(NUTDEVICE *dev, CONST void *data);
extern int NutDeviceWrite_P(NUTDEVICE *dev, PGM_P data, int len);
extern int NutDeviceIOCtl(NUTDEVICE *dev, int req, void *conf);

#ifndef __IMAGECRAFT__
#ifdef __AVR_ATmega128__
#define UDR     UDR0
#define UBRR    UBRR0L
#define USR     UCSR0A
#define UCR     UCSR0B
#define EICR    EICRB
#endif
#endif
#endif
