/*
 * Copyright (C) 2001-2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 * -
 * Portions Copyright (C) 2000 David J. Hudson <dave@humbug.demon.co.uk>
 *
 * This file is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You can redistribute this file and/or modify it under the terms of the GNU
 * General Public License (GPL) as published by the Free Software Foundation;
 * either version 2 of the License, or (at your discretion) any later version.
 * See the accompanying file "copying-gpl.txt" for more details.
 *
 * As a special exception to the GPL, permission is granted for additional
 * uses of the text contained in this file.  See the accompanying file
 * "copying-liquorice.txt" for details.
 * -
 * Portions Copyright (c) 1983, 1993 by
 *  The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *  This product includes software developed by the University of
 *  California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * -
 * Portions Copyright (c) 1993 by Digital Equipment Corporation.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies, and that
 * the name of Digital Equipment Corporation not be used in advertising or
 * publicity pertaining to distribution of the document or software without
 * specific, written prior permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

/*
 * $Log: ifconfig.c,v $
 * Revision 1.16  2002/07/03 16:41:32  harald
 * Bugfix: Failed to store gateway
 *
 * Revision 1.15  2002/06/26 17:29:36  harald
 * First pre-release with 2.4 stack
 *
 */

#include <string.h>

#include <eeprom.h>

#include <net/ether.h>
#include <net/route.h>
#include <arpa/inet.h>

#include <sys/confnet.h>

#include <pro/dhcp.h>

#ifdef NUTDEBUG
#include <sys/print.h>
#endif

/*!
 * \addtogroup xgIP
 */
/*@{*/

CONFNET confnet;

static u_char egnite_null_mac[] = { 0x00,0x06,0x98,0x00,0x00,0x00 };

/*!
 * \brief Load network configuration from EEPROM.
 *
 * \param name Name of the device.
 *
 * \return 0 if configuration has been read. Otherwise the
 *         return value is -1, IP address is 0, IP netmask
 *         is 255.255.255.0 and MAC is 00-06-98-00-00-00.
 */
int NutNetLoadConfig(CONST char *name)
{
    eeprom_read_block(&confnet, CONFNET_EE_OFFSET, sizeof(CONFNET));
    if(confnet.cd_size == sizeof(CONFNET) && strcmp(confnet.cd_name, name) == 0) 
        return 0;

    memcpy(confnet.cdn_mac, egnite_null_mac, 6);
    confnet.cdn_ip_addr = 0;
    confnet.cdn_ip_mask = inet_addr("255.255.255.0");
    confnet.cdn_cip_addr = 0;
    confnet.cdn_gateway = 0;
    return -1;
}

/*!
 * Save network configuration in EEPROM.
 */
int NutNetSaveConfig(void)
{
    u_char *cp;
    int i;

    confnet.cd_size = sizeof(CONFNET);
    for(cp = (u_char *)&confnet, i = 0; i < sizeof(CONFNET); i++) 
        eeprom_wb(i + CONFNET_EE_OFFSET, *cp++);

    return 0;
}

/*!
 * Network interface setup.
 *
 * \param dev  Identifies the network device to setup. This pointer must 
 *             have been retrieved previously by calling NutDeviceOpen().
 * \param ip_addr
 * \param ip_mask
 * \param gateway
 *
 * \return 0 on success, -1 otherwise.
 *
 * \note Applications typically do not call this function.
 *       It's automatically called by NutNetIfConfig() or
 *       indirectly by NutNetAutoConfig().
 */
int NutNetIfSetup(NUTDEVICE *dev, u_long ip_addr, u_long ip_mask, u_long gateway)
{
    IFNET *nif;

    nif = dev->dev_icb;

    /*
     * Use specified or previously used IP address.
     */
    if(ip_addr == 0 && (ip_addr = confnet.cdn_ip_addr) == 0)
        return -1;
    nif->if_local_ip = ip_addr;

    /*
     * Use specified or default mask.
     */
    if(ip_mask == 0)
        ip_mask = inet_addr("255.255.255.0");
    nif->if_mask = ip_mask;

    /*
     * Add routing entries.
     */
    NutIpRouteAdd(ip_addr & ip_mask, ip_mask, 0, dev);
    if(gateway)
        NutIpRouteAdd(0, 0, gateway, dev);

    /*
     * Save configuration in EEPROM.
     */
    memcpy(confnet.cd_name, dev->dev_name, sizeof(confnet.cd_name));
    memcpy(confnet.cdn_mac, nif->if_mac, sizeof(nif->if_mac));
    confnet.cdn_ip_addr = ip_addr;
    confnet.cdn_ip_mask = ip_mask;

    /*
     * Set gateway, if one was provided by the caller. Remove
     * gateway, if it's outside of our network.
     */
    if(gateway || (confnet.cdn_gateway & ip_mask) != (ip_addr & ip_mask))
        confnet.cdn_gateway = gateway;

    return NutNetSaveConfig();
}

/*!
 * \brief Configure a network interface.
 *
 * Open the device and associate the given MAC address, IP address and 
 * netmask with the network interface.
 *
 * \param name    Name of the device to configure. This device must have
 *                been registered by calling NutRegisterDevice().
 * \param mac     6-byte Ethernet MAC address to be used. This must be
 *                a unique address within the local Ethernet. The vendor
 *                of the Ethernut board will supply an address, which is
 *                unique within the known universe.
 * \param ip_addr IP address to be used in network byte order. This must 
 *                be a unique address within the Internet. If you do not 
 *                directly communicate with other Internet hosts, you can 
 *                use a locally assigned address.
 * \param ip_mask IP network mask to be used in network byte order. 
 *                Typical networks use 255.255.255.0, which allows upto 
 *                254 hosts.
 *
 * \return 0 on success, -1 otherwise.
 */
int NutNetIfConfig(CONST char *name, CONST u_char *mac, u_long ip_addr, u_long ip_mask)
{
    NUTDEVICE *dev;
    IFNET *nif;

    /*
     * Open device by name.
     */
    if((dev = NutDeviceOpen(name)) == 0) 
        return -1;

    /*
     * We support Ethernet only.
     */
    if(dev->dev_type != IFTYP_NET)
        return -1;
    nif = dev->dev_icb;
    if(nif->if_type != IFT_ETHER)
        return -1;
    nif->if_recv = NutEtherInput;

    /*
     * Initialize interface.
     */
    memcpy(nif->if_mac, mac, sizeof(nif->if_mac));
    if((*dev->dev_init)(dev)) 
        return -1;

    /*
     * If we got a valid IP address, create 
     * a route and return.
     */
    if(ip_addr & ip_mask) {
        NutNetIfSetup(dev, ip_addr, ip_mask, 0);
        return 0;
    }

    /*
     * Start DHCP client and wait for any result.
     */
    return NutDhcpClientStart(dev);
}


/*!
 * \brief Configure a network interface using EEPROM values.
 *
 * Loads IP address, netmask and MAC address from EEPROM and calls
 * NutNetIfConfig().
 *
 * \param name Name of the device.
 *
 * \return 0 on success, -1 otherwise.
 */
int NutNetAutoConfig(CONST char *name)
{
    NutNetLoadConfig(name);
    return NutNetIfConfig(name, confnet.cdn_mac, confnet.cdn_cip_addr, confnet.cdn_ip_mask);
}

/*@}*/
